/* 128x64 LCD Diagnostic Test Utility
 *   http://www.pjrc.com/
 *
 */

#include "paulmon2.h"
#include <stdio.h>	/* for printf_fast, etc */


volatile xdata at 0xFE04 unsigned char left_cmd;
volatile xdata at 0xFE05 unsigned char left_status;
volatile xdata at 0xFE06 unsigned char left_data;
volatile xdata at 0xFE07 unsigned char left_data_rd;
volatile xdata at 0xFE08 unsigned char right_cmd;
volatile xdata at 0xFE09 unsigned char right_status;
volatile xdata at 0xFE0A unsigned char right_data;
volatile xdata at 0xFE0B unsigned char right_data_rd;

#define left_command(cmd)  {while (left_status & 0x80) ; left_cmd = (cmd); }
#define right_command(cmd) {while (right_status & 0x80) ; right_cmd = (cmd); }
#define left_write(cmd)    {while (left_status & 0x80) ; left_data = (cmd); }
#define right_write(cmd)   {while (right_status & 0x80) ; right_data = (cmd); }

unsigned char left_read(void)
{
	while (left_status & 0x80) ;
	return left_data_rd;
}

unsigned char right_read(void)
{
	while (right_status & 0x80) ;
	return right_data_rd;
}

void status_report(unsigned char val, unsigned char not_present_val)
{
	if (val == not_present_val) {
		printf_fast("  LCD probably did not drive the bus lines during read (%x is addr LSB)\r\n", not_present_val);
	} else if (val & 0x4F) {
		printf_fast("  Illegal status value, bits 6,3,2,1,0 should be low - maybe some other chip responded to status read?\r\n");
	}
	if (val & 0x10) {
		printf_fast("  reset bit set, LCD may be stuck in reset, check reset signal\r\n");
	}
	if (val & 0x20) {
		printf_fast("  LCD reports \"off\" (bit 5 high), but it should be on\r\n");
	}
	if (val & 0x80) {
		printf_fast("  LCD reports \"busy\" (bit 7 high), but it should not be busy\r\n");
	}
	if (val == 0) {
		printf_fast("  good\r\n");
	}
}

code char *left_string =  "This is a string to write into the left side buffer memory";
code char *right_string = "And a totally different string that gets writting ot the right";


void main()
{
	unsigned char val, i, error_count, no_drive_count;
	code char *p;

	printf_fast("128x64 LCD Diagnostic\r\n");
	printf_fast("---------------------\r\n");

	// turn both sides of the LCD on
	left_command(0x3F);
	right_command(0x3F);

	// read left side status
	val = left_status;
	printf_fast("\r\nread status, left side = %x\r\n", val);
	status_report(val, 5);

	// read left side status
	val = right_status;
	printf_fast("\r\nread status, right side = %x\r\n", val);
	status_report(val, 9);

	// write data into left side display buffer
	printf_fast("\r\nwrite to left side display buffer\r\n  ");
	left_command(0xB8);
	left_command(0x40);	// these 2 set the point to beginning of display buffer
	p = left_string;
	for (i=0; i<20; i++) {
		printf("%d ", *p);
		left_write(*p++);
	}
	printf("\r\n");

	// read data back from left side display buffer
	printf_fast("\r\nread back from left side display buffer\r\n  ");
	left_command(0xB8);
	left_command(0x40);
	p = left_string;
	error_count = no_drive_count = 0;
	left_read();
	for (i=0; i<20; i++) {
		val = left_read();
		printf("%d ", val);
		if (val == 7) no_drive_count++;
		if (val != (unsigned char)(*p++)) error_count++;
	}
	printf("\r\n");
	if (error_count) {
		printf_fast("  error: %d of 20 did not read back correctly\r\n", error_count);
	}
	if (no_drive_count == 20) {
		printf_fast("  all 7s probably means the LCD did not drive the bus during reads\r\n");
	}

	// write data into right side display buffer
	printf_fast("\r\nwrite to right side display buffer\r\n  ");
	right_command(0xB8);
	right_command(0x40);	// these 2 set the point to beginning of display buffer
	p = right_string;
	for (i=0; i<20; i++) {
		printf("%d ", *p);
		right_write(*p++);
	}
	printf("\r\n");

	// read data back from right side display buffer
	printf_fast("\r\nread back from right side display buffer\r\n  ");
	right_command(0xB8);
	right_command(0x40);
	p = right_string;
	error_count = no_drive_count = 0;
	right_read();
	for (i=0; i<20; i++) {
		val = right_read();
		printf("%d ", val);
		if (val == 11) no_drive_count++;
		if (val != (unsigned char)(*p++)) error_count++;
	}
	printf("\r\n");
	if (error_count) {
		printf_fast("  error: %d of 20 did not read back correctly\r\n", error_count);
	}
	if (no_drive_count == 20) {
		printf_fast("  all 11s probably means the LCD did not drive the bus during reads\r\n");
	}

	// are there other tests that could be done?
	// if you have any ideas, please contact paul@pjrc.com

	printf_fast("\r\npress any key to reboot");
	pm2_cin();
	pm2_restart();  /* ESC to quit */
}






void putchar(char c)
{
	pm2_cout(c);
}



